package usingjsp;

import java.io.*;
import java.net.*;

/** A class to recursively visit JSP files and precompile them using
 *  a JSP engine
 */
public class Precompile
{
    public static FileFilter fileFilter = new PrecompileFileFilter();
    public URL rootURL;

    public Precompile(URL aRootURL)
    {
        rootURL = aRootURL;
    }

/** Precompiles an entire directory of files */
    public void precompileDirectory(File dir, String startDir)
    {
// Get all the files that are either directories or JSP files (the filter
// source code is at the bottom of this file)
        File[] files =  dir.listFiles(fileFilter);

        for (int i=0; i < files.length; i++)
        {
// If the current file is a directory, precompile its contents
            if (files[i].isDirectory())
            {
                precompileDirectory(files[i], startDir+"/"+dir.getName());
            }
            else
            {
// Otherwise precompile the current file
                precompileFile(files[i], startDir+"/"+dir.getName());
            }
        }
    }

    public void precompileFile(File file, String startDir)
    {
        try
        {
// Create the URL for precompiling
            URL precompileURL = new URL(rootURL, startDir+"/"+
                file.getName()+"?jsp_precompile=true");

// Create a URL connection to the JSP
            HttpURLConnection conn = (HttpURLConnection) precompileURL.
                openConnection();      

// Check the response code (this also forces the connection to contact
// the web server and precompile the JSP)
            int responseCode = conn.getResponseCode();

            System.out.println(startDir+file.getName()+": "+responseCode);
        }
        catch (Exception exc)
        {
            System.out.println("Exception: "+exc.toString()+
                " for "+startDir+file.getName());
        }
    }

    public static void main(String[] args)
    {
        if (args.length < 2)
        {
            System.out.println(
                "Please supply a base URL and a base directory");
            System.exit(0);
        }

        try
        {
            String startURL = args[0];
            File start = new File(args[1]);

            Precompile precomp = new Precompile(new URL(startURL));

            if (start.isDirectory())
            {
                precomp.precompileDirectory(start, "");
            }
            else
            {
                precomp.precompileFile(start, "");
            }
        }
        catch (Exception exc)
        {
            exc.printStackTrace();
        }
    }
}

class PrecompileFileFilter implements FileFilter
{
    public boolean accept(File f)
    {
        if (f.isDirectory()) return true;
        if (f.getName().endsWith(".jsp")) return true;
        return false;
    }
}
