package usingjsp;

import java.io.*;
import java.util.*;

/** Manages the meet file using the file locking mechanism. */
public class MeetManager2
{
    public static void updateMeet(Meet meet)
        throws IOException
    {
        FileLock lock = new FileLock("meets.dat");

        if (lock.lock())
        {
// Get all the available meets
            Vector meets = getMeets(lock);

// Look for one with the same name as this one
            for (int i=0; i < meets.size(); i++)
            {
                Meet currMeet = (Meet) meets.elementAt(i);

                if (currMeet.meetName.equalsIgnoreCase(meet.meetName))
                {
// Replace the old meet with the new version
                    meets.setElementAt(meet, i);
                    break;
                }
            }

// Copy the meets back into the save file
            setMeets(lock, meets);

            lock.unlock();
        }
        else
        {
            throw new IOException("Unable to lock Meet file");
        }
    }

    public static void deleteMeet(String meetName)
        throws IOException
    {
        FileLock lock = new FileLock("meets.dat");

        if (lock.lock())
        {
// Get a list of all the meets
            Vector meets = getMeets(lock);

            for (int i=0; i < meets.size(); i++)
            {
                Meet currMeet = (Meet) meets.elementAt(i);

// Look for the meet with the same name as the meet to be deleted
                if (currMeet.meetName.equalsIgnoreCase(meetName))
                {
// Remove the meet from the vector
                    meets.removeElementAt(i);
                    break;
                }
            }

// Copy the vector of meets back to the file
            setMeets(lock, meets);

            lock.unlock();
        }
        else
        {
            throw new IOException("Unable to lock Meet file");
        }
    }

    public static void addMeet(Meet newMeet)
        throws IOException
    {
        FileLock lock = new FileLock("meets.dat");

        if (lock.lock())
        {
// Get a list of all the meets
            Vector meets = getMeets(lock);

// Add the new meet to the vector
            meets.addElement(newMeet);

// Save the new list of meets back to the file
            setMeets(lock, meets);

            lock.unlock();
        }
        else
        {
            throw new IOException("Unable to lock Meet file");
        }
    }

/** Returns a vector of all the meets in the file */
    public static Vector getAllMeets()
        throws IOException
    {
        FileLock lock = new FileLock("meets.dat");

        if (lock.lock())
        {
            Vector v = getMeets(lock);

            lock.unlock();

            return v;
        }
        else
        {
            throw new IOException("Unable to lock Meet file");
        }
    }

/** Gets a vector containing all the meets */

    public static Vector getMeets(FileLock lock)
        throws IOException
    {
// Get an input stream to read the data file
        InputStream in = lock.getInputStream();

        try
        {

// Try to wrap an object input stream around the normal input stream
// If the file is empty, this operation throws an error, so in order to
// close the file properly, you still need a reference to the file, that's
// why the lock.getInputStream is performed ahead of time.
            ObjectInputStream obIn = new ObjectInputStream(in);

// Read in the vector of objects
            Vector v = (Vector) obIn.readObject();

            in.close();
            return v;
        }
        catch (Exception ignore)
        {
        }

// Make sure the file is closed
        if (in != null)
        {
            in.close();
        }

        return new Vector();
    }

    public static void setMeets(FileLock lock, Vector meets)
        throws IOException
    {
        ObjectOutputStream out = new ObjectOutputStream(
            lock.getRewriteOutputStream());

        out.writeObject(meets);

        out.close();
    }
}
