package usingjsp;

import java.sql.*;
import java.util.*;
import com.wutka.util.*;

/** A connection pool that creates a fixed pool of connections at
 *  startup and waits for connections if they aren't available.
 */
public class ConnectionPool implements IConnectionPool
{
    protected ResourcePool pool;
    protected String connectionURL;
    protected String userName;
    protected String password;

/** Creates a new connection pool that creates connections using
 *  the specified URL, user name and password */
    public ConnectionPool(String aConnectionURL,
        String aUserName, String aPassword, int numConnections)
        throws SQLException
    {
        connectionURL = aConnectionURL;
        userName = aUserName;
        password = aPassword;
        pool = new ResourcePool();

// Create a fixed number of connections and add them to the pool
        for (int i=0; i < numConnections; i++)
        {
            pool.addResource(DriverManager.getConnection(connectionURL,
                userName, password));
        }
    }

/** Gets a connection from the pool */
    public synchronized Connection getConnection()
        throws SQLException
    {
        return (Connection) pool.getResource();
    }

/** Returns a connection to the pool */
    public synchronized void releaseConnection(Connection conn)
        throws SQLException
    {
        pool.releaseResource(conn);
    }
}
