package usingjsp.problemreport;

import usingjsp.DatabaseObject;
import java.sql.*;
import java.io.*;
import java.util.*;

/** A data object that contains a problem report */
public class ProblemReport extends DatabaseObject
    implements java.io.Serializable
{
    public int problemId;
    public java.util.Date entered;
    public String system;
    public String subsystem;
    public String originator;
    public String description;
    public int priority;
    public String status;
    public java.util.Date estimatedCompletionDate;
    public Vector notes;

    public ProblemReport()
    {
    }

/** Creates an instance of ProblemReport from a result set */
    public ProblemReport(ResultSet results)
        throws SQLException
    {
        problemId = results.getInt("problem_id");
        entered = results.getTimestamp("entered");
        system = results.getString("system");
        subsystem = results.getString("subsystem");
        originator = results.getString("originator");
        description = results.getString("description");
        priority = results.getInt("priority");
        status = results.getString("status");
        estimatedCompletionDate = results.getDate("ecd");
        Object tempNotes = results.getObject("notes");
        if (tempNotes == null)
        {
            notes = new Vector(0);
        }
// The mySQL driver returns the notes object as an array of bytes. Some
// drivers may see the object as its original type.
        else if (tempNotes instanceof Vector)
        {
            notes = (Vector) results.getObject("notes");
        }
        else if (tempNotes instanceof byte[])
        {
            try
            {
// Convert the byte array into an object
                ObjectInputStream obIn =
                    new ObjectInputStream(new ByteArrayInputStream(
                        (byte[]) tempNotes));
                notes = (Vector) obIn.readObject();
                obIn.close();
            } catch (Exception exc) {
                exc.printStackTrace();
            }
        }
        else
        {
// This really indicates an error
            System.out.println("The notes object is "+tempNotes.getClass().getName());
        }
    }

    public int getProblemId() { return problemId; }
    public void setProblemId(int aProblemId) { problemId = aProblemId; }

    public java.util.Date getEntered() { return entered; }
    public void setEntered(java.util.Date enteredTime)
        { entered = enteredTime; }

    public String getSystem() { return system; }
    public void setSystem(String aSystem) { system = aSystem; }

    public String getSubsystem() { return subsystem; }
    public void setSubsystem(String aSubsystem) { subsystem = aSubsystem; }

    public String getOriginator() { return originator; }
    public void setOriginator(String anOriginator)
        { originator = anOriginator; }
    
    public String getDescription() { return description; }
    public void setDescription(String aDescription)
        { description = aDescription; }
    
    public int getPriority() { return priority; }
    public void setPriority(int aPriority) { priority = aPriority; }

    public String getStatus() { return status; }
    public void setStatus(String aStatus) { status = aStatus; }

    public java.util.Date getEstimatedCompletionDate()
        { return estimatedCompletionDate; }
    public void setEstimatedCompletionDate(java.util.Date anECD)
        { estimatedCompletionDate = anECD; }

    public Vector getNotes() { return notes; }
    public void setNotes(Vector newNotes) { notes = newNotes; }

    public String getTableName() { return "problem_report"; }

    public DatabaseObject createInstance(ResultSet results)
        throws SQLException
    {
        return new ProblemReport(results);
    }

/** Overrides the base insert method to grab the sequence number after the
 *  object has been inserted. */

    public int insert(Connection conn)
        throws SQLException
    {
        int numRows = super.insert(conn);

        problemId = getSequenceNumber(conn);

        return numRows;
    }

    public String getFieldList()
    {
        return "problem_id, entered, system, subsystem, originator, "+
            "description, priority, status, ecd, notes";
    }

    public String getInsertStatement()
    {
        return "insert into problem_report (problem_id, entered, "+
            "system, subsystem, originator, description, priority, "+
            "status, ecd, notes) values (?,?,?,?,?,?,?,?,?,?)";
    }

    public void prepareInsertStatement(PreparedStatement s)
        throws SQLException
    {
        s.setInt(1, 0);
        s.setTimestamp(2, new Timestamp(entered.getTime()));
        s.setString(3, system);
        s.setString(4, subsystem);
        s.setString(5, originator);
        s.setString(6, description);
        s.setInt(7, priority);
        s.setString(8, status);
        s.setDate(9, new java.sql.Date(
            estimatedCompletionDate.getTime()));
        s.setObject(10, notes);
    }

    public String getUpdateStatement()
    {
        return "update problem_report set entered=?, "+
            "system=?, subsystem=?, originator=?, description=?,"+
            " priority=?, status=?, ecd=?, notes=? where problem_id=?";
    }

    public void prepareUpdateStatement(PreparedStatement s)
        throws SQLException
    {
        s.setTimestamp(1, new Timestamp(entered.getTime()));
        s.setString(2, system);
        s.setString(3, subsystem);
        s.setString(4, originator);
        s.setString(5, description);
        s.setInt(6, priority);
        s.setString(7, status);
        s.setDate(8, new java.sql.Date(
            estimatedCompletionDate.getTime()));
        s.setObject(9, notes);
        s.setInt(10, problemId);
    }

    public String getDeleteStatement()
    {
        return "delete from problem_report where problem_id=?";
    }

    public void prepareDeleteStatement(PreparedStatement s)
        throws SQLException
    {
        s.setInt(1, problemId);
    }

    public String getSequenceGenerator()
    {
        return "problem_id_gen";
    }
}
