package usingjsp.service;

import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.sql.*;
import java.util.*;
import usingjsp.IConnectionPool;
import usingjsp.ConnectionPool;

public class ServiceRequestsImpl extends UnicastRemoteObject
    implements ServiceRequests
{
    protected IConnectionPool pool;

    public ServiceRequestsImpl()
        throws RemoteException
    {
// Force the JDBC driver to load
        try
        {
            Class.forName("org.gjt.mm.mysql.Driver").newInstance();
        }
        catch (Exception exc)
        {
            throw new RemoteException(
                "Error loading JDBC driver: "+exc.toString());
        }

// Create the connection pool
        try
        {
            pool = new ConnectionPool(
                "jdbc:mysql://localhost/usingjsp",
                "", "", 10);
        }
        catch (SQLException exc)
        {
            throw new RemoteException(
                "Error creating connection pool: "+exc.toString());
        }
    }

/** Returns an array of all the service request */
    public RemoteServiceRequest[] getAll()
        throws RemoteException
    {
        return doQuery(null);
    }

/** Returns an array of all service request with a
 *  specific originator */
    public RemoteServiceRequest[] getAllByOriginator(String originator)
        throws RemoteException
    {
        return doQuery("originator = '"+originator+"'");
    }

/** Locates a service request by its request id */
    public RemoteServiceRequest getByRequestId(int requestId)
        throws RemoteException
    {
        RemoteServiceRequest[] reqs = doQuery("request_id="+requestId);

        if (reqs.length == 1) return reqs[0];
        return null;
    }

/** Creates a new, empty service request */
    public RemoteServiceRequest create()
        throws RemoteException
    {
        return new RemoteServiceRequestImpl(pool);
    }

/** Performs the actual database query (using DatabaseObject.getAll) */
    protected RemoteServiceRequest[] doQuery(String whereClause)
        throws RemoteException
    {
        Connection conn = null;
        
        try
        {

// Allocate a connection from the pool
            conn = pool.getConnection();

// Create a dummy object for performing a query
            ServiceRequest query = new ServiceRequest();

// Perform the query
            Vector v = query.getAll(conn, whereClause);

// Create an array to hold the results
            RemoteServiceRequest remotes[] = new
                RemoteServiceRequest[v.size()];

// Loop through the results
            for (int i=0; i < remotes.length; i++)
            {
// Wrap each ServiceRequest object with a remote version 
                remotes[i] = new RemoteServiceRequestImpl(
                    (ServiceRequest) v.elementAt(i), pool);
            }

            return remotes;
        }
        catch (SQLException exc)
        {
            throw new RemoteException(
                "Got exception while performing query: "+exc.toString());
        }
        finally
        {
            if (conn != null)
            {
                try
                {
                    pool.releaseConnection(conn);
                }
                catch (Exception ignore) {}
            }
        }
    }

    public static void main(String[] args)
    {
// Must assign an RMI security manager
        if (System.getSecurityManager() == null)
        {
            System.setSecurityManager(new RMISecurityManager());
        }

        try
        {
// Create an instance of the Service Request service
            ServiceRequestsImpl impl = new ServiceRequestsImpl();

// Register the service with the RMI registry
            Naming.rebind("//localhost/ServiceRequests", impl);

        }
        catch (Exception exc)
        {
            exc.printStackTrace();
        }
    }
}
