package usingjsp.motd;

import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.CORBA.*;

import java.io.*;

public class MOTDImpl extends _MOTDImplBase
{
/** The name of the file containing the message */
    protected String motdFileName;

/** Creates a new instance of the MOTD server assigned to a
 *  specific filename.
 */
    public MOTDImpl(String aMotdFileName)
    {
        motdFileName = aMotdFileName;
    }

    public String getMOTD()
    {
        try
        {
// Open the message file
            BufferedReader reader = new BufferedReader(
                new FileReader(motdFileName));

// Create a buffer to hold the resulting message
            StringBuffer buffer = new StringBuffer();

            String line;

// Read all the lines in the file
            while ((line = reader.readLine()) != null)
            {
// Put each line into the buffer separated by newlines
                buffer.append(line);
                buffer.append("\n");
            }

            reader.close();

// Return the message
            return buffer.toString();
        }
        catch (IOException exc)
        {
// If there's an error, just return an empty message
            return "";
        }
    }

    public static void main(String[] args)
    {
        String motdFile = System.getProperty("file", "motd.txt");
        String motdService = System.getProperty("service",
            "MessageOfTheDay");
        try
        {
// Create a reference to the CORBA orb
            ORB orb = ORB.init(args, null);

// Create an instance of the MOTD implementation
            MOTDImpl impl = new MOTDImpl(motdFile);

// Register the MOTD implementation with the ORB
            orb.connect(impl);
            
// Locate the CORBA naming service
            org.omg.CORBA.Object objRef =
                orb.resolve_initial_references("NameService");
               
// Cast the CORBA Object to NamingContext using the narrow method
            NamingContext ncRef = NamingContextHelper.narrow(objRef);

// Create a NameComponent representing the path for the MOTD service
            NameComponent nc = new NameComponent(motdService, "");
            NameComponent[] path = new NameComponent[] { nc };

// Register the MOTD implementation with the naming service
            ncRef.rebind(path, impl);

// Go into an infinite loop while the ORB services requests
            java.lang.Object sync = new java.lang.Object();
            synchronized (sync)
            {
                sync.wait();
            }
        }
        catch (Exception exc)
        {
            exc.printStackTrace();
        }
    }
}
