package usingjsp.xsl;

import java.io.*;

import javax.servlet.*;
import javax.servlet.http.*;

import org.apache.xalan.xslt.*;
import org.w3c.dom.*;
import org.xml.sax.*;

public class XSLServlet extends HttpServlet
{
    public void service(HttpServletRequest request,
        HttpServletResponse response)
        throws IOException, ServletException
    {

// Get the XML source passed in the request
        Object source = request.getAttribute("source");

        XSLTInputSource xmlSource = null;

// See if the source is an XML document
        if (source instanceof Document)
        {
            xmlSource = new XSLTInputSource((Document) source);
        }
// Otherwise see if it is a file or a filename
        else if ((source instanceof File) || (source instanceof String))
        {
            if (source instanceof File)
            {
                xmlSource = new XSLTInputSource(
                    new FileReader((File) source));
            }
            else
            {
                xmlSource = new XSLTInputSource(
                    new FileReader((String) source));
            }
        }

// Get the URL for the stylesheet (assume for now that it's local)
        String stylesheet = (String) request.getAttribute("stylesheet");

        String stylesheetPath = null;

// If the request came from a wireless application, pull the stylesheet
// from the wml directory, otherwise use the html directory
        if (isWML(request))
        {
            stylesheetPath = "wml"+File.separator+stylesheet;
            response.setContentType("text/vnd.wap.wml");
        }
        else
        {
            stylesheetPath = "html"+File.separator+stylesheet;
            response.setContentType("text/html");
        }

// Create an input source for reading the stylesheet (use the servlet
// context to get an input stream for the stylesheet)

        XSLTInputSource stylesheetSource = new XSLTInputSource(
            getServletContext().getResourceAsStream(stylesheetPath));

        
// Create a result target that writes to the servlet's output stream
        XSLTResultTarget target = new XSLTResultTarget(response.getWriter());

// Create an XSL processor and process the XML source and the stylesheet
        try
        {
            XSLTProcessor processor = XSLTProcessorFactory.getProcessor();

            processor.process(xmlSource, stylesheetSource, target);
        }
        catch (SAXException exc)
        {
            throw new ServletException(exc);
        }
    }

    public static boolean isWML(HttpServletRequest request)
    {
// Checks to see if the request came from a wireless app (from chapter 26)
        String accept = request.getHeader("Accept");
        if (accept == null) return false;

        if (accept.indexOf("text/vnd.wap.wml") >= 0)
        {
            return true;
        }

        return false;
    }
}
