package usingjsp.scores;

import usingjsp.DatabaseObject;
import java.sql.*;
import java.io.*;
import java.util.*;

/** A class to contain scores */
public class Score extends DatabaseObject
    implements java.io.Serializable
{
    public int round;
    public String player;
    public String opponent;
    public int playerScore;
    public int opponentScore;

    public Score()
    {
    }

    public Score(ResultSet results)
        throws SQLException
    {
        round = results.getInt("round");
        player = results.getString("player");
        opponent = results.getString("opponent");
        playerScore = results.getInt("player_score");
        opponentScore = results.getInt("opponent_score");
    }

    public int getRound() { return round; }
    public void setRound(int aRound) { round = aRound; }

    public String getPlayer() { return player; }
    public void setPlayer(String aPlayer) { player = aPlayer; }

    public String getOpponent() { return opponent; }
    public void setOpponent(String aOpponent)
        { opponent = aOpponent; }

    public int getPlayerScore() { return playerScore; }
    public void setPlayerScore(int aPlayerScore)
        { playerScore = aPlayerScore; }

    public int getOpponentScore() { return opponentScore; }
    public void setOpponentScore(int aOpponentScore)
        { opponentScore = aOpponentScore; }

/** Returns the name of the table containing scores */
    public String getTableName() { return "scores"; }

/** Returns a list of the table fields this class uses */
    public String getFieldList()
    {
        return "round,player,opponent,player_score,opponent_score";
    }

/** Creates a new instance of this object from a database row */
    public DatabaseObject createInstance(ResultSet results)
        throws SQLException
    {
        return new Score(results);
    }

/** Returns the SQL statement used to insert this object into
 *  the database */
    public String getInsertStatement()
    {
        return "insert into scores "+
            "(round,player,opponent,player_score,opponent_score) "+
            " values (?,?,?,?,?)";
    }

/** Populates an insert statement with this object's data */
    public void prepareInsertStatement(PreparedStatement s)
        throws SQLException
    {
        s.setInt(1, round);
        s.setString(2, player);
        s.setString(3, opponent);
        s.setInt(4, playerScore);
        s.setInt(5, opponentScore);
    }

/** Returns the SQL statement used to update this object in
 *  the database */
    public String getUpdateStatement()
    {
        return "update scores set "+
            "opponent=?,player_score=?,opponent_score=? "+
            " where round=? and player=?";
    }

/** Populates an update statement with this object's data */
    public void prepareUpdateStatement(PreparedStatement s)
        throws SQLException
    {
        s.setString(1, opponent);
        s.setInt(2, playerScore);
        s.setInt(3, opponentScore);
        s.setInt(4, round);
        s.setString(5, player);
    }

/** Returns the SQL statement used to delete this object from
 *  the database */
    public String getDeleteStatement()
    {
        return "delete from scores where round=? and player=?";
    }

/** Populates a delete statement with this object's data */
    public void prepareDeleteStatement(PreparedStatement s)
        throws SQLException
    {
        s.setInt(1, round);
        s.setString(2, player);
    }
}
