package usingjsp;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import com.sun.image.codec.jpeg.*;
import javax.servlet.*;
import javax.servlet.http.*;

public class EmbossImageServlet extends HttpServlet
{
    public void doGet(HttpServletRequest request,
        HttpServletResponse response)
        throws IOException
    {
// Make sure the browser calls this servlet every time
        response.setHeader("Cache-Control", "no-cache");
        response.setHeader("Pragma", "no-cache");
        response.setHeader("Expires", "0");

        HttpSession session = request.getSession();

// Get the in-memory image
        BufferedImage buffImage =
            (BufferedImage) session.getAttribute("image");
        
// If there's no image, create a blank one
        if (buffImage == null)
        {
            buffImage = new BufferedImage(100, 100,
                BufferedImage.TYPE_INT_RGB);
        }

// Create a blank image to hold the embossed image
        BufferedImage outputImage =
            new BufferedImage(buffImage.getWidth(),
                buffImage.getHeight(), BufferedImage.TYPE_INT_RGB);

// Perform the emboss
        embossImage(buffImage, outputImage);

// Create an encoder to convert the image into JPEG format, using
// the servlet's output stream as the stream for writing the image
        JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(
            response.getOutputStream());

// Convert the image to JPeg
        encoder.encode(outputImage);

    }

// To emboss an image, you start with a completely gray destination image.
// For each pixel in the source image, look pixels to the upper-left and
// lower-right. Figure out the change in red, green, and blue between the
// upper-left and lower-right and look at the maximum change (either maximum
// positive or maximum negative) for any color component. For example,
// if the green changed by -5, blue changed by 10 and red changed by
// -100, the maximum change would be -100 (the red, which changed the most).
//
// Now, add the amount of change to 128 (the gray level) and create a
// pixel in the destination image with red, green, and blue values equal
// to the new gray level. Make sure you adjust the gray level so it can't
// be less than 0 or more than 255.
//
// 
    public void embossImage(BufferedImage srcImage, BufferedImage destImage)
    {

        int width = srcImage.getWidth();
        int height = srcImage.getHeight();

// Loop through every pixel

        for (int i=0; i < height; i++) {
            for (int j=0; j < width; j++) {

// Assume that the upper left and lower right are 0
                int upperLeft = 0;
                int lowerRight = 0;

// If the pixel isn't on the upper or left edge, get the upper-left
// pixel (otherwise, the upper-left for edge pixels is the default of 0)

                if ((i > 0) && (j > 0)) {

// The & 0xffffff strips off the upper 8 bits, which is the transparency

                    upperLeft = srcImage.getRGB(j-1, i-1) 
                        & 0xffffff;
                }

// If the pixel isn't on the bottom or right edge, get the lower-righto
// pixel (otherwise, the lower-right for egde pixels is the default of 0)

                if ((i < height-1) && (j < width-1)) {

// The & 0xffffff strips off the upper 8 bits, which is the transparency
                    lowerRight = srcImage.getRGB(j+1, i+1) 
                        & 0xffffff;
                }

// Get the differences between the red, green and blue pixels

                int redDiff = ((lowerRight >> 16) & 255) -
                    ((upperLeft >> 16) & 255);
                int greenDiff = ((lowerRight >> 8) & 255) -
                    ((upperLeft >> 8) & 255);
                int blueDiff = (lowerRight & 255) -
                    (upperLeft & 255);

// Figure out which color had the greatest change

                int diff = redDiff;
                if (Math.abs(greenDiff) > Math.abs(diff)) 
                    diff=greenDiff;
                if (Math.abs(blueDiff) > Math.abs(diff)) 
                    diff=blueDiff;


// Add the greatest change to a medium gray
                int greyColor = 128 + diff;

// If the gray is too high or too low, make it fit in the 0-255 range
                if (greyColor > 255) greyColor = 255;
                if (greyColor < 0) greyColor = 0;

// Create the new color, and don't forget to add in a transparency
// of 0xff000000 making the image completely opaque

                int newColor = 0xff000000 + (greyColor << 16) +
                    (greyColor << 8) + greyColor;

                destImage.setRGB(j, i, newColor);
            }
        }
    }
}


