package usingjsp;

import java.awt.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.io.*;
import com.sun.image.codec.jpeg.*;
import javax.servlet.*;
import javax.servlet.http.*;

public class LightingEffectsServlet extends HttpServlet
{
    public void doGet(HttpServletRequest request,
        HttpServletResponse response)
        throws IOException
    {
// Make sure the browser calls this servlet every time
        response.setHeader("Cache-Control", "no-cache");
        response.setHeader("Pragma", "no-cache");
        response.setHeader("Expires", "0");

        HttpSession session = request.getSession();

// Get the in-memory image
        BufferedImage buffImage =
            (BufferedImage) session.getAttribute("image");
        
// If there's no image, create a blank one
        if (buffImage == null)
        {
            buffImage = new BufferedImage(100, 100,
                BufferedImage.TYPE_INT_RGB);
        }

// Get the lighting parameters
        int centerX = Integer.parseInt(request.getParameter("centerX"));
        int centerY = Integer.parseInt(request.getParameter("centerY"));
        int radius = Integer.parseInt(request.getParameter("radius"));
        double intensity = Double.parseDouble(
            request.getParameter("intensity"));
        double fade = Double.parseDouble(
            request.getParameter("fade"));
        
// Can't have an intensity less than 0, an intensity of 0 is total darkness
// "How much blacker could it be? The answer is 'none more black'." -N.Tufnel
        if (intensity < 0.0) intensity = 0.0;

        int width = buffImage.getWidth();
        int height = buffImage.getHeight();

// Create a blank image to hold the new filtered image
        BufferedImage outputImage = new BufferedImage(
            width, height, BufferedImage.TYPE_INT_RGB);
            
// Loop through all the pixels in the image
        for (int x=0; x < width; x++)
        {
            for (int y=0; y < height; y++)
            {
// Get the current pixel value, compute its lighting value
                int rgb = buffImage.getRGB(x, y);
                rgb = filterRGB(x, y, rgb, centerX, centerY, radius,
                    intensity, fade);

// .. then copy it to the new image
                outputImage.setRGB(x, y, rgb);
            }
        }
        
        response.setContentType("image/jpeg");

// Create an encoder to convert the image into JPEG format, using
// the servlet's output stream as the stream for writing the image
        JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(
            response.getOutputStream());

// Convert the image to JPeg
        encoder.encode(outputImage);

    }

    public int filterRGB(int x, int y, int rgb, int centerX,
        int centerY, int radius, double intensity, double fade)
    {
// Save the pixel's transparency value
        int trans = rgb & 0xff000000;

// Compute the distance from the edge of the circle (distance from center
// - radius).
        double dist = Math.sqrt((x-centerX)*(x-centerX) +
            (y-centerY)*(y-centerY)) - radius;
        if (dist < 0.0) dist = 0.0;

// Compute the intensity based on distance and fade
        double intense = intensity - dist * fade;

// Again, none more black than 0.0
        if (intense < 0.0) intense = 0.0;

// Adjust the colors based on the new intensity
        int red = (int)(((rgb >> 16) & 0xff) * intense);
// Max color value for each component is 255
        if (red > 255) red = 255;

        int green = (int)(((rgb >> 8) & 0xff) * intense);
        if (green > 255) green = 255;

        int blue = (int)((rgb & 0xff) * intense);
        if (blue > 255) blue = 255;

// Return the new color
        return trans + (red << 16) + (green << 8) + blue;
    }
}
