import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import java.io.*;

/** Converts an XML file with JSP syntax into a regular JSP file */

public class XMLToJSP implements EntityResolver
{
    public XMLToJSP()
    {
    }

/** Looks for references to the JSP dtd which Sun has not put on their
 *  web site even though it's in the JSP spec. Substitutes a local
 *  copy of the JSP DTD. */
    public InputSource resolveEntity(String publicId, String systemId)
    {
        if (systemId.startsWith("http://java.sun.com/products/jsp/dtd"))
        {
            try
            {
                return new InputSource(new FileReader("jsp.dtd"));
            }
            catch (IOException exc)
            {
                exc.printStackTrace();
            }
            return null;
        }
        else
        {
            return null;
        }
    }

    public static void main(String[] args)
    {
        if (args.length < 1)
        {
            System.out.println("Please supply an XML file");
        }

// Figure out the root name of the file
// read from zzz.xml, write to zzz.jsp
        String xmlFileName = args[0];
        if (!xmlFileName.toLowerCase().endsWith(".xml"))
        {
            xmlFileName = xmlFileName + ".xml";
        }
        String jspFileName = xmlFileName.substring(0,
            xmlFileName.indexOf(".xml"))+".jsp";

        try
        {
// Create a DocumentBuilderFactory
            DocumentBuilderFactory fact =
                DocumentBuilderFactory.newInstance();
        
// Create a Document Builder to parse the XML file
            DocumentBuilder builder = fact.newDocumentBuilder();

// Set up an entity resolver to weed out reference to the
// phantom JSP DTD mentioned in the JSP spec
            builder.setEntityResolver(new XMLToJSP());
            
// Open the XML input file
            BufferedReader xmlFile =
                new BufferedReader(new FileReader(
                    xmlFileName));
        
// Parse the input file
            Document doc = builder.parse(
                new InputSource(xmlFile));

            xmlFile.close();

            if (doc == null)
            {
                    System.out.println("No document available");
                    System.exit(1);
            }

            PrintWriter out = new PrintWriter(
                new FileWriter(jspFileName));

// Write out the JSP file from the XML document object
            writeJSPChildren(doc, out);

            out.close();

        } catch (Exception exc) {
            exc.printStackTrace();
        }
    }

    public static void writeJSP(Node n, PrintWriter out)
    {
// If this is a text node, print out its value
        if ((n.getNodeType() == Node.CDATA_SECTION_NODE) ||
            (n.getNodeType() == Node.TEXT_NODE))
        {
            CharacterData textNode = (CharacterData) n;
            out.print(textNode.getData());
            return;
        }
// Ignore everything else that isn't an element or a document
        else if ((n.getNodeType() != Node.ELEMENT_NODE) &&
                (n.getNodeType() != Node.DOCUMENT_NODE))
        {
            return;
        }

// Get the name of the tag
        String nodeName = n.getNodeName();

// For JSP root, look for xmlns tags and create <%@ taglib directives
        if (nodeName.equals("jsp:root"))
        {
            NamedNodeMap attrs = n.getAttributes();
            
            int numAttrs = attrs.getLength();
            
            for (int i=0; i < numAttrs; i++)
            {
                Node attr = attrs.item(i);

                if (attr.getNodeName().startsWith("xmlns:"))
                {
                    String ns = attr.getNodeName().substring(6);
                    if (ns.equals("jsp")) continue;

                    out.println("<%@ taglib uri=\""+attr.getNodeValue()+
                        "\" prefix=\""+ns+"\" %>");
                }
            }

            writeJSPChildren(n, out);
        }

// For a directive, get the directive name then print out the attributes
        else if (nodeName.startsWith("jsp:directive."))
        {
            out.print("<%@ "+nodeName.substring(14));
            NamedNodeMap attrs = n.getAttributes();

            int numAttrs = attrs.getLength();

            for (int i=0; i < numAttrs; i++)
            {
                Node attr = attrs.item(i);

                out.print(" "+attr.getNodeName()+"=\""+
                    attr.getNodeValue()+"\"");
            }
            out.println(" %>");
        }
// For a scriptlet, just print out the <% %> tags
        else if (nodeName.equals("jsp:scriptlet"))
        {
            out.println("<%");
            writeJSPChildren(n, out);
            out.println("%>");
        }
// For an expression, just print out the <%= %> tags
        else if (nodeName.equals("jsp:expression"))
        {
            out.print("<%= ");
            writeJSPChildren(n, out);
            out.print(" %>");
        }
// For a declaration, just print out the <%! %> tags
        else if (nodeName.equals("jsp:declaration"))
        {
            out.println("<%!");
            writeJSPChildren(n, out);
            out.println("%>");
        }
// For anything else, assume it's part of the document and print it out
        else
        {
            out.print("<"+nodeName);
            NamedNodeMap attrs = n.getAttributes();

            int numAttrs = attrs.getLength();

            for (int i=0; i < numAttrs; i++)
            {
                Node attr = attrs.item(i);

                out.print(" "+attr.getNodeName()+"=\""+
                    attr.getNodeValue()+"\"");
            }
            out.print(">");
            writeJSPChildren(n, out);
            out.print("</"+nodeName+">");
        }
    }

// Write out the elements contained within a particular node
    public static void writeJSPChildren(Node n, PrintWriter out)
    {
        NodeList children = n.getChildNodes();

        int numChildren = children.getLength();

        for (int i=0; i < numChildren; i++)
        {
            writeJSP(children.item(i), out);
        }
    }
}
